var insertElementSequenceUrl = "{% url 'core_composer_insert_element_sequence' %}";
var changeXsdTypeUrl = "{% url 'core_composer_change_xsd_type' %}";
var renameElementUrl = "{% url 'core_composer_rename_element' %}";
var deleteElementUrl = "{% url 'core_composer_delete_element' %}";
var getElementOccurrencesUrl = "{% url 'core_composer_get_element_occurrences' %}";
var setElementOccurrencesUrl = "{% url 'core_composer_set_element_occurrences' %}";
var saveTemplateUrl = "{% url 'core_composer_save_template' %}";
var saveTypeUrl = "{% url 'core_composer_save_type' %}";
var changeRootTypeNameUrl = "{% url 'core_composer_change_root_type_name' %}";
var composerIndexUrl = "{% url 'core_composer_index' %}";
