{% extends 'core_main_app/admin/templates/versions/available.html' %}

{% block box_tools %}

{% if not data.version_manager.user %}
	<a href="{% url 'admin:core_composer_app_upload_type_version' data.version_manager.id %}"
	   class="pull-right btn btn-default">
		<i class="fa fa-upload"></i> Upload New Version
	</a>
{% endif %}

{% url data.back_url as type_list_url %}
{% include 'core_main_app/common/buttons/go_to.html' with url=type_list_url label='Back to Types' %}

{% endblock %}