{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}save-type-modal{% endblock %}
{% block modal_title %}Save Type{% endblock %}

{% block modal_body %}
<label for="newTypeName">Name:</label>
<input type="text" class="form-control" id="newTypeName"/>
<div id="new-type-error" class="text-danger">
</div>
{% endblock %}
{% block modal_footer %}
<a class="btn btn-default pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Cancel</a>
<a id="save-type" class="btn btn-primary" href="#"><i class="fa fa-floppy"></i>Save</a>
{% endblock %}
