<ul id='menu_element' class='options_menu'>
    <li><span class="menu rename-element">Rename</span></li>
    <li><span class="menu element-occurrences">Manage Occurrences</span></li>
    <li><span class="menu delete-element">Delete</span></li>
</ul>