{% extends 'core_main_app/common/templates/versions/available.html' %}

{% block box_actions %}
   {% if 'core_parser_app' in INSTALLED_APPS %}
    <a class="btn btn-default modules"
        href="{% url data.module_url version.object %}">
        <i class="fa fa-cubes"></i> Modules
    </a>
   {% endif %}
{% endblock %}