{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}change-element-type-modal{% endblock %}
{% block modal_title %}Change Element Type{% endblock %}

{% block modal_body %}
Select the new type:
<div>
    <select id="newXSDtype">
        <option value="sequence">Sequence</option>
        <option value="choice">Choice</option>
    </select>
</div>
{% endblock %}
{% block modal_footer %}
<a class="btn btn-default pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Cancel</a>
<a id="change-element-type" class="btn btn-primary" href="#">Update</a>
{% endblock %}
